<?php
function srps_settings_panels(){
	return apply_filters( 'srps_settings_panels', array() );
}

function srps_settings_fields_types(){
	return apply_filters( 'srps_settings_fields_types', array() );
}

/**
 * Add a section to a settings panel
 *
 * Add a section to a settings panel
 *
 */
class Srps_Settings_Panel_Add_Section{
	public $panel_id;
	public $section_id;
	public $title;
	public $description;

	public function __construct( $panel_id, $section_id, $title = false, $description = '', $priority = 20 ){
		$this->panel_id    = $panel_id;
		$this->section_id  = $section_id;
		$this->title       = $title;
		$this->description = $description;

		add_filter( $this->panel_id . '_panel_sections', array($this, 'add'), $priority );
	}

	public function add( $sections ){
		if( ! array_key_exists($this->section_id, $sections) ){
			$sections[ $this->section_id ] = array(
				'title' => $this->title,
				'description' => $this->description,
			);
		}
		return $sections;
	}
}

/**
 * Add a field to a settings panel
 *
 * Add a field to a settings panel
 *
 */
class Srps_Settings_Panel_Add_Field{
	public $panel_id;
	public $section_id;
	public $field_id;
	public $field_type;
	public $settings;
	public $description;

	public function __construct( $panel_id, $section_id, $field_id, $field_type = 'input', $settings = array(), $priority = 20 ){
		$this->panel_id    = $panel_id;
		$this->section_id  = $section_id;
		$this->field_id    = $field_id;
		$this->field_type  = $field_type;
		$this->settings    = $settings;

		add_filter( $this->panel_id . '_panel_fields', array($this, 'add'), $priority );
		add_action( $section_id. '_srps_fields', array($this, 'render'), $priority );
	}

	public function getSettings(){
		$defaults = array(
			'title' => '',
			'description' => '',
			'default' => '',
		);
		$settings = (array) $this->settings;
		$settings['field_type'] = $this->field_type;
		return wp_parse_args( $settings, $defaults );
	}

	public function add( $fields ){
		if( ! array_key_exists($this->field_id, (array) $fields) ){
			$fields[ $this->field_id ] = $this->getSettings();
		}
		return $fields;
	}

	public function getField(){
		$settings     = $this->getSettings();
		$known_fields = srps_settings_fields_types();
		$type = $settings['field_type'];
		if( array_key_exists($type, $known_fields) ){
			if( isset( $known_fields[ $type ] ) && class_exists( $known_fields[ $type ] ) ){
				$class = $known_fields[ $type ];
				if( method_exists($class, 'render') ){
					$final_obj = new $class( $this->panel_id, $this->field_id, $settings );
					return $final_obj->render();
				}
			}
		}
	}

	public function render(){
		$s = $this->getSettings();
		$description  = !empty($s['description']) ? '<p class="description">'. $s['description'] .'</p>' : '';

		echo '<tr id="'. $this->panel_id .'_'. $this->field_id .'">
			<th scope="row">'. $s['title'] .'</th>
			<td>'. $this->getField() . $description .'</td>
		</tr>';
	}
}

//------------------------------------//--------------------------------------//

class Srps_Settings_Panel_Init{
	public $id;
	public $title;
	public $settings;

	public function __construct( $id, $title = '', $settings = array() ){
		$this->id       = $id;
		$this->title    = ! empty($title) ? $title : $id;
		$this->settings = $settings;

		add_action('admin_menu', array( $this, 'menu' ) );
		add_filter('srps_settings_panels', array( $this, 'panels' ) );
		add_action( 'admin_init', array( $this, 'panelInit' ) );
	}

	public function getSettings(){
		return wp_parse_args(
			$this->settings,
			array(
				'parent' => 'edit.php?post_type=smk_recipes',
				'capability' => 'manage_options',
			)
		);
	}

	public function menu(){
		$sett = $this->getSettings();

		add_submenu_page( 
			$sett['parent'], 
			$this->title, 
			$this->title, 
			$sett['capability'], 
			$this->id, 
			array($this, 'panelPage') 
		);
	}

	/**
	 * Admin panel initialization
	 *
	 * Register setting and init the panel.
	 * Note: this DO NOT render the admin page.
	 *
	 */
	public function panelInit() {
		register_setting( $this->id, $this->id, array($this, 'sanitize') );
	}

	/**
	 * Sanitize data
	 *
	 * Sanitize user input before saving it into DB. Uses the sanitize method from each field type.
	 *
	 * @param array $value All fields in array, key pairs. field_id => field_value(user input)
	 * @return array Sanitized fields. 
	 */
	public function sanitize( $value ){
		$all_registered_fields = $this->fields();
		$all_registered_field_types = srps_settings_fields_types();
		$new_value = $value;
	
		if( is_array( $value ) ){
			foreach ($value as $field_id => $field_value) {

				if( array_key_exists($field_id, $all_registered_fields) ){
					
					// Get field settings
					if( ! empty( $all_registered_fields[ $field_id ] ) ){
						$field_settings = $all_registered_fields[ $field_id ];
					}
					else{
						$new_value[ $field_id ] = $field_value;
						continue;
					}

					// Get field type
					if( !empty($field_settings['field_type']) ){
						$field_type = $field_settings['field_type'];
					}
					else{
						$new_value[ $field_id ] = $field_value;
						continue;
					}

					// Get field class
					if( array_key_exists($field_type, $all_registered_field_types) ){
						$class = $all_registered_field_types[ $field_type ];
					}
					else{
						$new_value[ $field_id ] = $field_value;
						continue;
					}

					// Sanitize
					if( class_exists($class) ){
						$field_class = new $class( $this->id, $field_id, $field_settings );
						$new_value[ $field_id ] = $field_class->sanitize( $field_value );
					}
				}
			}
		}

		return $new_value;
	}

	/**
	 * All registered panels
	 *
	 *All registered panels
	 *
	 * @return array All panels `id => title` 
	 */
	public function panels( $panels ){		
		if( ! array_key_exists($this->id, $panels) ){
			$panels[ $this->id ] = $this->getSettings();
		}
		return $panels;
	}

	/**
	 * Panel sections
	 *
	 * All active panel sections
	 *
	 * @return array All sections `id => settings` 
	 */
	public function sections(){
		return apply_filters( $this->id . '_panel_sections', array(
			'general' => array(
				'title'       => __('General', 'smk_theme'),
				'description' => '',
			)
		) );
	}

	/**
	 * Panel fields
	 *
	 * All active panel fields
	 *
	 * @return array All fields `id => settings` 
	 */
	public function fields(){
		return apply_filters( $this->id . '_panel_fields', array() );
	}

	public function panelPage(){
		$this->pageOpen();

		settings_fields( $this->id );
		
		// Iterate sections
		foreach ($this->sections() as $section_id => $section_settings) {

			// If the section has fields show it otherwise hide.
			if( has_action( $section_id. '_srps_fields' ) ){
				
				// Section title and description
				$this->sectionTitle( $section_settings['title'] );
				$this->sectionDescription( $section_settings['description'] );

				// Show the fields for this section
				echo '<table class="form-table"><tbody>';
					do_action( $section_id. '_srps_fields' );
				echo '</tbody></table>';
			}
		}
		$this->pageClose();
		// $this->pageDebug();
	}

	public function pageOpen(){
		$output = '<div class="wrap" id="recipes-panel-'. $this->id .'">';
		$output .= '<form method="post" action="options.php">';
		$output .= '<h2>'. __('Recipes settings', 'smk_theme') .'</h2>';
		echo $output;
	}

	public function pageClose(){
		$output = submit_button();
		$output .= '</form>';
		$output .= '</div>';
		echo $output;
	}

	public function sectionTitle( $title ){
		echo apply_filters( 'srps_settings_section_title', '<br /><hr /><br /><h3>'. $title .'</h3>', $title );
	}

	public function sectionDescription( $description ){
		if( !empty($description) ){
			echo apply_filters( 'srps_settings_section_description', '<p>'. $description .'</p>', $description );
		}
	}

	public function updateOption( $key, $value ){
		$options = get_option( $this->id );
		$options = ( is_array($options) ) ? $options : array();
		$new_options = wp_parse_args(
			array( $key, $value ),
			$options
		);
		update_option( $this->id, $new_options );
	}

	public function update(){
		if( !empty($_GET['page']) && $_GET['page'] == $this->id && isset($_GET['update-settings']) ){

		}
	}

	public function pageDebug(){

		// DANGEROUS!!! IT WILL DELETE ALL SAVED SETTINGS
		// delete_option( $this->id );

		if( !empty($_POST) ){
			echo '<pre>';
			print_r( $_POST );
			echo '</pre>';
		}
		echo '<pre>';
			print_r( $this->sections() );
			print_r( $this->fields() );
			print_r( get_option( $this->id ) );
		echo '</pre>';
	}

}

//------------------------------------//--------------------------------------//

/**
 * Add settings panel
 *
 * Create a settings panel or add fields and sections to an existing one
 *
 * @param $panel_id The panel ID. Required 
 * @param $panel_settings The panel settings. Optional 
 * @return void 
 */
class Srps_Settings_Panel{
	public $panel_id;
	public $section;

	public function __construct( $panel_id, $title = '', $settings = array()  ){
		$this->panel_id = $panel_id;
		$this->section  = 'general';
		$all_panels = srps_settings_panels();
		if( ! array_key_exists($panel_id, (array) $all_panels) ){
			new Srps_Settings_Panel_Init( $panel_id, $title, $settings );
		}
	}

	public function addField( $field_id, $field_type = 'input', $settings = array(), $priority = 20 ){
		new Srps_Settings_Panel_Add_Field($this->panel_id, $this->section, $field_id, $field_type, $settings, $priority );
	}
	
	public function addSection( $section_id, $section_title = false, $description = '', $priority = 20 ){
		// Add new section
		if( !empty($section_title) && trim($section_title) !== false ){
			new Srps_Settings_Panel_Add_Section($this->panel_id, $section_id, $section_title, $description, $priority);
		}
		
		// Switch to this section
		$this->section = $section_id;
	}

}


class Srps_Register_Field_Type_Obj{
	public $type;
	public $class;

	public function __construct( $type, $class ){
		$this->type = $type;
		$this->class = $class;
		add_filter('srps_settings_fields_types', array($this, 'add'));
	}

	public function add( $types ){
		if( is_array( $types ) && ! array_key_exists( $this->type, $types) ){
			$types[ $this->type ] = $this->class;
		}
		return $types;
	}
}

function srps_register_field_type( $type, $class ){
	new Srps_Register_Field_Type_Obj( $type, $class );
}


//------------------------------------//--------------------------------------//

/**
 * Field type base class
 *
 * Define field type base class. This class is used only as parent and can't be initiated.
 *
 * @param $field_id The field ID. Required 
 * @param $settings The field settings. Optional 
 * @return void 
 */
abstract class Srps_Settings_Field_Type{
	public $id;
	public $panel_id;
	public $settings;

	public function __construct( $panel_id, $field_id, $settings = array() ){
		$this->id       = $field_id;
		$this->panel_id = $panel_id;
		$this->settings = $settings;
	}

	public function defaultSettings(){
		return array();
	}

	public function getSettings(){
		return wp_parse_args( $this->settings, $this->defaultSettings() );
	}

	public function getSetting( $setting_key ){
		$settings = $this->getSettings();
		return ( isset($settings[ $setting_key ]) ) ? $settings[ $setting_key ] : false;
	}

	public function getFieldName(){
		return esc_attr( $this->panel_id .'['. $this->id .']' );
	}

	public function getFieldValue(){
		$option = get_option( $this->panel_id );
		if( !empty($option) ){
			if( isset($option[ $this->id ]) ){
				return $option[ $this->id ];
			}
		}
		else{
			return !empty($this->settings['default']) ? $this->settings['default'] : '';
		}
	}

	public function sanitize( $value ){
		return $value;
	}

	abstract public function render();
}

/*
-------------------------------------------------------------------------------
Text
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Text extends Srps_Settings_Field_Type{

	public function defaultSettings(){
		return array(
			'allow_safe_html' => false,
			'size' => 'regular',
			'type_attr' => 'text',
		);
	}

	public function render(){
		$size = $this->getSetting( 'size' );
		$type_attr = $this->getSetting( 'type_attr' );
		$type  = 'text';
		$size_class = 'regular-text';

		// Input type
 		if( !empty($type_attr) ){
			if( in_array( $type_attr, array('text', 'password', 'email', 'number', 'url', ) ) ){
				$type = $type_attr;
			}
		}

		// Input width
		if( !empty($size) ){
			if( in_array( $size, array('wide', 'widefat', 'large') ) ){
				$size_class = 'widefat';
			}
			elseif( in_array( $size, array('small', 'small-text', 'mini') ) ){
				$size_class = 'small-text';
			}
			elseif( 'none' == $size ){
				$size_class = '';
			}
		}

		return '<input type="'. $type .'" value="'. esc_html( $this->getFieldValue() ) .'" name="'. $this->getFieldName() .'" class="'. $size_class .'" />';
	}

	public function sanitize( $value ){
		$allow_safe_html = $this->getSetting('allow_safe_html');
		return ( !empty($allow_safe_html) ) ? wp_kses_data( $value ) : sanitize_text_field( $value );
	}
}
srps_register_field_type( 'text', 'Srps_Settings_Field_Text' );

/*
-------------------------------------------------------------------------------
Textarea
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Textarea extends Srps_Settings_Field_Type{

	public function defaultSettings(){
		return array(
			'rows' => 5,
			'size' => 'regular',
			'allow_html' => true, // 'safe', 'any' or 'all' or 'unfiltered', 'limited'
		);
	}

	public function render(){
		$size = $this->getSetting( 'size' );
		$rows = absint( $this->getSetting( 'rows' ) );
		$rows = $rows > 0 ? $rows : 5;
		$size_attr = '';

		// Input width
		if( !empty($size) && ($size = trim( $size )) ){
			if( in_array( $size, array('wide', 'widefat', 'large') ) ){
				$size_attr = ' class="widefat"';
			}
			elseif( is_numeric( $size ) ){
				$size_attr = ' cols="'. absint( $size ) .'"';
			}
		}

		return '<textarea name="'. $this->getFieldName() .'"'. $size_attr .' rows="'. $rows .'">'. esc_textarea( $this->getFieldValue() ) .'</textarea>';
	}

	public function sanitize( $value ){
		$allow_html = $this->getSetting('allow_html');

		// Sanitize
		if( 'safe' == $allow_html ){
			$value = wp_kses_post( $value ); // Link in post
		}
		elseif( 'limited' == $allow_html ){
			$value = wp_kses_data( $value ); // Only some inline tags
		}
		elseif( in_array( $allow_html, array('any', 'all', 'unfiltered') ) ){
			$value = $value; // Any HTML tags and attr, even 'script'. RAW
		}
		elseif( $allow_html === false ){
			$value = strip_tags( $value ); // No tags allowed at all
		}
		else{
			$value = wp_kses_post( $value ); // Default. Can use only the tags that are allowed in posts.
		}

		return $value;
	}
}
srps_register_field_type( 'textarea', 'Srps_Settings_Field_Textarea' );

/*
-------------------------------------------------------------------------------
Select
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Select extends Srps_Settings_Field_Type{
	public function render(){
		return '<select name="'. $this->getFieldName() .'"></select>';
	}
}
srps_register_field_type( 'select', 'Srps_Settings_Field_Select' );

/*
-------------------------------------------------------------------------------
Page select
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Page_Select extends Srps_Settings_Field_Type{

	public function defaultSettings(){
		return array(
			'show_empty_label' => true,
		);
	}

	public function render(){
		$show_empty_label = $this->getSetting( 'show_empty_label' );
		$option_none = null;
		if( !empty( $show_empty_label ) ){
			if( true !== $show_empty_label ){
				$option_none = $show_empty_label;
			}
			else{
				$option_none = __('-- Select page --', 'smk_theme');
			}
		} 

		$args = array(
			'depth'                 => 0,
			'child_of'              => 0,
			'selected'              => absint( $this->getFieldValue() ),
			'echo'                  => 0,
			'name'                  => $this->getFieldName(),
			'id'                    => null,
			'show_option_none'      => $option_none,
			'show_option_no_change' => null,
			'option_none_value'     => null,
		);
		return wp_dropdown_pages( $args );
	}

	public function sanitize( $value ){
		$page_exists = ( absint( $value ) ) ? get_post( absint( $value ) ) : false;
		return ( !empty($page_exists) ) ? absint( $value ) : '';
	}
}
srps_register_field_type( 'page_select', 'Srps_Settings_Field_Page_Select' );

/*
-------------------------------------------------------------------------------
Checkbox
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Checkbox extends Srps_Settings_Field_Type{

	public function defaultSettings(){
		return array(
			'display_inline' => false,
		);
	}

	public function render(){
		$options        = $this->getSetting( 'options' );
		$display_inline = $this->getSetting( 'display_inline' );
		$value          = $this->getFieldValue();

		$output = '<input name="'. $this->getFieldName() .'" type="hidden" value="" />';

		// If is multicheckbox.
		if( !empty($options) && is_array($options) ){
			foreach ($options as $option_value => $option_label) {
				$checked = ( in_array($option_value, (array) $value ) ) ? ' checked="checked"' : '';
				$inline  = ( !empty($display_inline) ) ? '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' : '<br />';
				$output .= '<label><input type="checkbox" value="'. $option_value .'" name="'. $this->getFieldName() .'[]"'. $checked .' />'. esc_html( $option_label ) .'</label>'. $inline;
			}
		}

		// Else is single checkbox with 'on' value
		else{
			$output .= '<label><input type="checkbox" value="on" name="'. $this->getFieldName() .'" '. checked( $this->getFieldValue(), 'on', false ) .' />'. $this->getSetting( 'label' ) .'</label>';
		}

		return $output;
	}
}
srps_register_field_type( 'checkbox', 'Srps_Settings_Field_Checkbox' );

/*
-------------------------------------------------------------------------------
Checkbox
-------------------------------------------------------------------------------
*/
class Srps_Settings_Field_Radio extends Srps_Settings_Field_Type{

	public function defaultSettings(){
		return array(
			'display_inline' => false,
		);
	}

	public function render(){
		$options        = $this->getSetting( 'options' );
		$display_inline = $this->getSetting( 'display_inline' );
		$value          = $this->getFieldValue();

		$output = '<input name="'. $this->getFieldName() .'" type="hidden" value="" />';

		if( !empty($options) && is_array($options) ){
			foreach ($options as $option_value => $option_label) {
				$checked = ( in_array($option_value, (array) $value ) ) ? ' checked="checked"' : '';
				$inline  = ( !empty($display_inline) ) ? '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' : '<br />';
				$output .= '<label><input type="radio" value="'. $option_value .'" name="'. $this->getFieldName() .'"'. $checked .' />'. esc_html( $option_label ) .'</label>'. $inline;
			}
		}

		else{
			$output .= '<label><input type="radio" value="on" name="'. $this->getFieldName() .'" '. checked( $this->getFieldValue(), 'on', false ) .' />'. __('On', 'smk_theme') .'</label>'. $inline;
			$output .= '<label><input type="radio" value="off" name="'. $this->getFieldName() .'" '. checked( $this->getFieldValue(), 'off', false ) .' />'. __('Off', 'smk_theme') .'</label>'. $inline;
		}

		return $output;
	}
}
srps_register_field_type( 'radio', 'Srps_Settings_Field_Radio' );


/**
 * Get a option from DB
 *
 * Get the saved options from DB or a single option from the multidimensional array.
 *
 * @param string $panel_id The registered panel. It's the ID registered with Srps_Settings_Panel( $id, ... )
 * @param string $option_name The field ID. It's the ID registered with $panel->addField( $id, ... )
 * @param string $default Default value to show if the option is not saved in DB.
 * @return mixed 
 */
function srps_get_option( $panel_id, $option_name = false, $default = false ){
	$options = get_option( $panel_id );
	
	// If is the user does not indicate the option name to retrieve, return all options.
	if( empty($option_name) ){
		return $options;
	}

	// If the option is saved return its value.
	if( isset( $options[ $option_name ] ) ){
		return $options[ $option_name ];
	}

	// If the option is not saved yet and a default is set by user return it.
	elseif( $default !== false ){
		return $default;
	}

	// If all above fails, return false. This means, the option is not saved in DB.
	else{
		return false;
	}
}