<?php
/**
 * Update recipes in admin area
 *
 * Designed to to update the DB every time the Smk Recipes plugin is updated
 *
 */

class Smk_Recipes_Update_Admin{
	
	/**
	 * The Smk Recipes version saved in DataBase
	 *
	 * @return string 
	 */
	public $dbVersion;

	/**
	 * The current Smk Recipes version
	 *
	 * @return string 
	 */
	public $currentVersion = '1.5';

	/**
	 * Last updated version. Got in update process
	 *
	 * @return string 
	 */
	public $lastVersion = 0;

	function __construct(){
		if( $this->isAdministrator() ){
			$this->dbVersion = get_option( 'smk_recipes_version' );
			$this->dbVersion = !empty( $this->dbVersion ) ? $this->dbVersion : 0;
			
			add_action( 'admin_notices', array( $this, 'requireUpdate' ) );
			add_action( 'init', array( $this, 'processUpdate' ), 999 );
		}
	}

	function requireUpdate(){
		if( ! isset( $_GET['smk-recipes-update-nag'] ) ){ 
			if( version_compare($this->currentVersion, $this->dbVersion) > 0){
				$notice = '<div class="update-nag"><p>';
					$notice .= sprintf( 
						__( 'An update is required for <%1$s>Recipes</%1$s>. <%2$s>Click here to update now.</%3$s>. It can take a few minutes. Please be patient.', 'smk_theme' ),
						'strong',
						'a href="'. esc_url( add_query_arg( array('smk-recipes-update-nag' => 'true') ) ) .'" class="smk-recipes-admin-update-nag-process"',
						'a'
					);
				$notice .= '</p></div>';
				echo $notice;
			}
		}
		else{
			if( version_compare($this->currentVersion, $this->dbVersion) == 0){
				$notice = '<div class="updated"><p>';
					$notice .= sprintf( 
						__( '<%1$s>Recipes</%1$s> have been updated. Now your site is ready to use the new improvements. Thank you.', 'smk_theme' ),
						'strong'
					);
				$notice .= '</p></div>';
				echo $notice;
			}
		}
	}

	function processUpdate(){
		// Current user can process the update
		if( $this->canProcessUpdate() ){
				
			set_time_limit( 360 ); // 6 minutes

			//Process all updates for each version
			if( $this->updateFor( '1.0.1.1' ) ){
				$this->updateMetaAlreadyPublished();
			}
			if( $this->updateFor( '1.0.1.2' ) ){
				$this->updateIngredientsTax();
			}

			if( $this->updateFor( '1.0.1.3' ) ){
				$this->updateFeaturedMeta();
			}
			if( $this->updateFor( '1.1' ) ){
				$this->migrateSettingsPanel();
			}
			if( $this->updateFor( '1.2' ) ){
				$this->update1_2();
			}
			if( $this->updateFor( '1.2.1' ) ){
				$this->update1_2_1();
			}
				
			if( $this->updateFor( '1.2.2' ) ){
				$this->update1_2_2();
			}
			
			if( $this->updateFor( '1.3' ) ){
				$this->update1_3();
			}				
			
			if( $this->updateFor( '1.5' ) ){
				$this->update1_5();
			}
				
			// Update DB Recipes Version.
			$this->dbUpdate();
		}
	}

	function updateFor( $version ){
		$this->lastVersion = $version;
		return ( version_compare($this->currentVersion, $version) >= 0 && version_compare($version, $this->dbVersion) > 0 );
	}

	function dbUpdate(){
		if( $this->lastVersion !== 0 ){
			update_option( 'smk_recipes_version', $this->lastVersion );
		}
	}

	function isAdministrator(){
		return ( is_admin() && current_user_can('manage_options') );
	}

	function canProcessUpdate(){
		return ( !empty( $_GET['smk-recipes-update-nag'] ) && $this->isAdministrator() );
	}

	/*
	-------------------------------------------------------------------------------
	Helpers
	-------------------------------------------------------------------------------
	*/
	/**
	 * Update already published meta
	 *
	 * If the recipe has "published" status update the meta "smk_recipe_already_published" to "yes"
	 *
	 * @return void 
	 */
	public function updateMetaAlreadyPublished(){
		$query = array (
			'post_type'      => 'smk_recipes',
			'posts_per_page' => -1,
			'post_status'    => 'publish',
			'meta_query' => array(
				array(
					'key'     => 'smk_recipe_already_published',
					'compare' => 'NOT EXISTS',
				),
			),
		);
		$recipes_query = new WP_Query($query);
		if ($recipes_query->have_posts()) {
			while ($recipes_query->have_posts()) : $recipes_query->the_post();
				update_post_meta( get_the_ID(), 'smk_recipe_already_published', 'yes' );
			endwhile; 
		}
	}

	/**
	 * Get all ingredients and update the taxonomy
	 *
	 * In older versions the ingredients were stored only in post meta, add them now in a special taxonomy
	 *
	 * @return void 
	 */
	public function updateIngredientsTax(){
		$query = array (
			'post_type'      => 'smk_recipes',
			'posts_per_page' => -1,
			'post_status'    => 'any',
		);
		$recipes_query = new WP_Query($query);
		if ($recipes_query->have_posts()) {
			while ($recipes_query->have_posts()) : $recipes_query->the_post();
				$post_id = get_the_ID();
				
				$all_ingredients = get_post_meta( $post_id, 'smk_recipe_ingredients', true );

				$ingredients_tax = array();
				if( !empty( $all_ingredients[0] ) && is_array($all_ingredients[0]) ){
					foreach ($all_ingredients[0] as $ingr_index => $ingredient) {
						if( !empty($ingredient['ingredient']) ){
							$ingredients_tax[] = $ingredient['ingredient'];
						}
					}
					$new_ingredients = array();
					$new_ingredients[0] = $all_ingredients[0];
					update_post_meta( $post_id, 'smk_recipe_ingredients', $new_ingredients );
				}

				wp_set_object_terms( $post_id, $ingredients_tax, 'recipe_ingredients', false );

			endwhile; 
		}
		wp_reset_postdata();
	}

	/**
	 * Update all recipes that are or not featured
	 *
	 * Update all recipes that are or not featured and set the according value and time
	 *
	 * @return void 
	 */
	public function updateFeaturedMeta(){
		$query = array (
			'post_type'      => 'smk_recipes',
			'posts_per_page' => -1,
			'post_status'    => 'any',
		);
		$recipes_query = new WP_Query($query);
		if ($recipes_query->have_posts()) {
			while ($recipes_query->have_posts()) : $recipes_query->the_post();
				$post_id = get_the_ID();
				
				$is_featured   = get_post_meta( $post_id, 'smk_recipe_is_featured', true );
				$featured_time = get_post_meta( $post_id, 'smk_recipe_featured_time', true );

				if( empty( $is_featured ) ){
					update_post_meta( $post_id, 'smk_recipe_is_featured', 'no' );
					update_post_meta( $post_id, 'smk_recipe_featured_time', '0' );
				}
				elseif( 'yes' == $is_featured ){
					update_post_meta( $post_id, 'smk_recipe_is_featured', 'yes' );
					if( empty($featured_time) ){
						update_post_meta( $post_id, 'smk_recipe_featured_time', '1' );
					}
				}

			endwhile; 
		}
		wp_reset_postdata();
	}

	/**
	 * Send all saved data from the old panel to the new panel
	 *
	 * Send all saved data from the old panel to the new panel
	 *
	 * @return void 
	 */
	public function migrateSettingsPanel(){
		$old = get_option('smkop_smk_recipes_settings');
		$new_data = array();
		
		$new_data['submit_recipe_page'] = !empty($old['submit_recipe_page']) ? $old['submit_recipe_page'] : '';
		$new_data['recipe_submission_enabled'] = !empty($old['recipe_submission_enabled']) ? 'on' : '';
		$new_data['recipe_measurements'] = !empty($old['recipe_measurements']) ? $old['recipe_measurements'] : '';

		$new_pages = array(
			'smk_recipe_search' => array( 'recipe_search_page', __('Search recipes', 'smk_theme') ),
			'smk_members_list' => array( 'members_list_page', __('Members', 'smk_theme') ),
			'smk_tags_index' => array( 'tags_index_page', __('Tags index', 'smk_theme') ),
			// 'smk_register_user_form' => array( 'registration_page', __('Registration', 'smk_theme') ),
			// 'smk_signin_user_form' => array( 'signin_page', __('Sign In', 'smk_theme') ),
		);

		if( empty($new_data['submit_recipe_page']) ){
			$new_pages['smk_submit_recipe'] = array( 'submit_recipe_page', __('Submit recipe', 'smk_theme') );
		}

		$new_ids = array();
		foreach ($new_pages as $shortcode => $sett) {		
			$the_page = array(
				'post_title'    => wp_strip_all_tags( $sett[1] ),
				'post_content'  => '['. $shortcode .']',
				'post_status'   => 'publish',
				'post_type'     => 'page',
			);
			
			if( $shortcode == 'smk_members_list' ){
				$the_page_id = $this->getPageTemplateID( 'tpl-members.php' );
				if( !empty( $the_page_id ) ){
					$the_page['ID'] = $the_page_id;
				}
			}
			elseif( $shortcode == 'smk_tags_index' ){
				$the_page_id = $this->getPageTemplateID( 'tpl-tags-index.php' );
				if( !empty( $the_page_id ) ){
					$the_page['ID'] = $the_page_id;
				}
			}
			elseif( $shortcode == 'smk_submit_recipe' ){
				$the_page_id = $this->getPageTemplateID( 'tpl-submit-recipe.php' );
				if( !empty( $the_page_id ) ){
					$the_page['ID'] = $the_page_id;
				}
			}

			// Insert the post into the database
			$page_id = wp_insert_post( $the_page );
			$new_ids[ $sett[0] ] = $page_id;
		}

		if( !empty($new_ids) ){
			$new_data = wp_parse_args( $new_ids, $new_data);
		}

		update_option('smk_recipes_main_settings', $new_data);
		delete_option('smkop_smk_recipes_settings');
		
	}

	public function update1_2(){
		set_theme_mod( 'show_header_search_icon', 1 );
		set_theme_mod( 'show_header_pm_icon', 1 );
		set_theme_mod( 'show_header_submission_icon', 1 );
		set_theme_mod( 'show_header_user_menu', 1 );
	}

	public function update1_2_1(){
		$options = get_option('smk_recipes_main_settings');
		
		$options['recipe_submission_required_fields'] = array('featured_image', 'ingredients', 'tags', 'prep_mode');
		$options['ui_enable_filter'] = 'on';
		$options['ui_enable_favorites'] = 'on';
		$options['ui_enable_likes'] = 'on';
		$options['ui_enable_print'] = 'on';
		$options['ui_enable_views'] = 'on';
		$options['add_to_fav_points'] = 10;
		$options['sanction_points_for_remove_fav'] = -2;
		$options['like_recipe_points'] = 10;
		$options['sanction_points_for_dislike'] = -2;
		$options['publish_recipe_points'] = 50;
		$options['recipes_per_page'] = 12;
		$options['default_recipes_view'] = 'large';
		$options['recipes_view_grid_number_of_columns'] = 3;

		update_option('smk_recipes_main_settings', $options);
	}

	public function update1_2_2(){
		$options = get_option('smk_recipes_main_settings');
		
		$options['recipe_submission_enabled_fields'] = array(
			'featured_image', 'ingredients', 'tags', 'prep_mode', 'difficulty', 'prep_time', 'footnotes', 'links'
		);

		update_option('smk_recipes_main_settings', $options);
	}

	public function update1_3(){
		$options = get_option('smk_recipes_main_settings');
		
		$options['recipe_submission_min_image_width'] = 400;
		$options['ui_enable_ratings'] = 'on';
		$options['admin_list_unapproved_recipes'] = 'on';

		update_option('smk_recipes_main_settings', $options);
	}

	public function update1_5(){
		$options = get_option('smk_recipes_main_settings');
		
		$options['registered_only_can_view_recipes'] = 'off';

		update_option('smk_recipes_main_settings', $options);
	}

	public function getPageTemplateID( $tpl ){
		$pages_args = array(
			'meta_key' => '_wp_page_template',
			'meta_value' =>  $tpl
		);

		$pages = get_pages( $pages_args );
		if( !empty($pages) ){
			$page = array_shift($pages);
			return $page->ID;
		}
		else{
			return false;
		}
	}

}
$upd = new Smk_Recipes_Update_Admin;