<?php	
/**
 * User form base
 *
 * This class is instatiated only once. Here are added all fields and they are accessible via
 * static methods. Also here the class adds the fields to user profile.
 *
 * @return void 
 */
class Smk_User_Form_Base{
	
	public function __construct(){
	 	add_action( 'show_user_profile', array( $this, 'display' ) );
		add_action( 'edit_user_profile', array( $this, 'display' ) );
		add_action( 'personal_options_update', array( $this, 'update' ) );
		add_action( 'edit_user_profile_update', array( $this, 'update' ) );
	}
		
	/**
	 * Display
	 *
	 * Display all registered fields and sections in the admin form
	 * 
	 * @action-hook show_user_profile
	 * @action-hook edit_user_profile
	 *
	 * @param object $user The user object  
	 * @return string The HTML formated table with all sectons and fields 
	 */
	public function display( $user ){
		$sections = $this->sections();
		if( !empty( $sections ) ){

			foreach ($sections as $section_id => $section_settings) {
				$title = !empty($section_settings['title']) ? $section_settings['title'] : $section_id;
				
				if( !empty($section_settings['capability']) ){
					if( ! current_user_can( $section_settings['capability'] ) )
						continue;
				}

				echo '<h3>'. $title . '</h3>';
				echo '<table class="form-table">';
					do_action( 'smk_user_fields' . $section_id, $user );
				echo '</table>';
			}

		}
	}
		
	/**
	 * Update the form
	 *
	 * Update the form fields on submit
	 *
	 * @action-hook personal_options_update
	 * @action-hook edit_user_profile_update
	 *
	 * @param string $user_id The user id where the data should be updated 
	 */
	function update( $user_id ){
		$fields = $this->fields();
		if( isset($_POST) ){
			if( !empty($fields) ){
				foreach ($fields as $field_id => $field_settings) {
					if( isset( $_POST[ $field_id ] ) ){
						
						if( !empty($field_settings['capability']) ){
							if( ! current_user_can( $field_settings['capability'] ) )
								continue;
						}

						$data = $_POST[ $field_id ];
						$types = Smk_User_Form_Base::types();

						if( array_key_exists($field_settings['type'], $types) ){
							$class = $types[ $field_settings['type'] ];
							if( class_exists($class) ){
								$obj = new $class( $field_id, $field_settings );
								$data = $obj->sanitize( $data, $field_settings );
							}
						}

						update_user_meta( $user_id, $field_id, $data );
					}
				}
			}
		}
	}

	/**
	 * All sections
	 *
	 * Get all registered sections
	 *  
	 * @return array / section_id => section_settings 
	 */
	public static function sections(){
		return apply_filters('smk_user_form_sections', array());
	}

	/**
	 * All fields
	 *
	 * Get all registered fields
	 *  
	 * @return array / field_id => field_settings 
	 */
	public static function fields(){
		return apply_filters('smk_user_form_fields', array());
	}

	/**
	 * All field types
	 *
	 * Get all registered field types
	 *  
	 * @return array / field_type => field_class_name 
	 */
	public static function types(){
		return apply_filters('smk_user_form_field_types', array());
	}
}

//------------------------------------//--------------------------------------//
		
/**
 * Add field
 *
 * Adds the field defined by user. This is used in the main form class
 *
 * @param string $id The field ID  
 * @param array  $settings The field settings  
 * @param string $section The section where the field will be appended
 * @return void
 */
class Smk_User_Form_AddField{

	// Field id
	public $id;

	// Field settings
	public $settings;

	public function __construct( $id, $settings = array(), $section = '' ){
		$this->id = $id;
		$this->settings = wp_parse_args( $settings, array(
			'title' => $id,
		));

		// If the field is not registered, register it and display.
		if( !array_key_exists( $this->id, Smk_User_Form_Base::fields() ) ){
			add_filter( 'smk_user_form_fields', array( $this, 'registerField' ) );
			add_action( 'smk_user_fields' . $section, array($this, 'displayField') );
		}
	}
	
	/**
	 * Register
	 *
	 * Register this user field
	 *
	 * @filter-hook smk_user_form_fields
	 *
	 * @param array $prev Previously registered fields array  
	 * @return array 
	 */
	public function registerField( $prev ){
		$new[ $this->id ] = $this->settings;
		return wp_parse_args( $new, $prev );
	}

	/**
	 * Display
	 *
	 * Display the field HTML
	 *
	 * @param object $user  
	 * @return string The HTML 
	 */
	public function displayField( $user ){
		$settings = $this->settings;
		$types = Smk_User_Form_Base::types();
		if( array_key_exists($this->settings['type'], $types) ){
			$class = $types[ $this->settings['type'] ];
			if( class_exists($class) ){
				$html = new $class( $this->id, $this->settings );

				if( !empty($this->settings['capability']) ){
					if( ! current_user_can( $this->settings['capability'] ) )
						return;
				}

				echo $html->display( $user );
			}
		}
	}
}

//------------------------------------//--------------------------------------//
		
/**
 * Main form class
 *
 * This is used by user, to init the object and make accessible the methods to add the fields.
 * Also this accepts section ID and section settings. If the section is already registered, 
 * the settings will be ignored and the fields will be appended to the already registered section.
 *
 * @param string $section Section ID  
 * @param array  $section_settings Section settings  
 * @return void 
 */
if( ! class_exists( 'Smk_User_Form' )){
	class Smk_User_Form{
		
		// Section ID
		public $section;

		// Section settings
		public $settings;

		public function __construct( $section, $section_settings = array() ){
			$this->section = $section;
			$this->settings = wp_parse_args( $section_settings, array(
				'title' => $section,
			));

			// If section is not registered
			if( !array_key_exists( $this->section, Smk_User_Form_Base::sections() ) ){
				add_filter( 'smk_user_form_sections', array( $this, 'registerSection' ) );
			}
		}

		/**
		 * Register
		 *
		 * Register this section
		 *
		 * @filter-hook smk_user_form_sections
		 *
		 * @param array $prev Previously registered sections array  
		 * @return array 
		 */
		public function registerSection( $prev ){
			$new[ $this->section ] = $this->settings;
			return wp_parse_args( $new, $prev );
		}

		/**
		 * Add field
		 *
		 * Add a field to this section. This method is for public use.
		 *
		 * @param string $id The field ID. Must be unique. If it is already registered it will be ignored.  
		 * @param array $settings The field settings.  
		 * @return void 
		 */
		public function addField( $id , $settings = array() ){
			$input = new Smk_User_Form_AddField( $id, $settings, $this->section );
		}


	}
}