<?php
/**
 * Smk Page Builder Module
 *
 * Base module class designed to be extended to create custom page builder modules.
 * Browse `default` folder for examples. 
 *
 * -------------------------------------------------------------------------------------
 * Author: Smartik
 * Author URI: http://smartik.ws/
 * Copyright: (c) 2014 Smartik. All rights reserved
 * -------------------------------------------------------------------------------------
 *
 */
if( ! class_exists('Smk_PageBuilder_Module') ) {
	abstract class Smk_PageBuilder_Module {

		public $icon = array(
					'class' => 'file4',
					'color' => '#777',
				);
		public $iconSuffix = '-module-icon';

		var $id;    //Page builder id.
		var $mode;  //Page builder mode. 'create' menu, 'load' form, 'render' data.

		################################################################################

		/**
		 * Module construct
		 *
		 * @param string $mode `create`, `load`
		 */
		public function __construct($mode = ''){
			$this->id = 'smk_page_builder';
			$this->mode = $mode;
			if( 'create' == $this->mode ){
				add_action( $this->id . '_module_menu', array( &$this, 'addModuleMenu' ) );
				add_action( $this->id  .'_modules_icon_css', array( &$this, 'printIconCss' ) );
			}
		}

		################################################################################

		/**
		 * Form options array
		 *
		 * @return array
		 */
		abstract public function form();

		################################################################################

		/**
		 * Render module on front-end
		 *
		 * @param array $data The saved data.
		 * @return string Content to display on front-end
		 */
		abstract public function render($data);

		################################################################################

		/**
		 * Default settings
		 *
		 * @return array
		 */
		public function settings(){
			return array(
				'title'    => __('Default module', 'smk'),
				'icon'     => $this->icon,
				'desc'    => __('Just for testing', 'smk'),
				'min-grid' => 1,
				'category' => '',
			);
		}

		################################################################################

		/**
		 * Print Icon CSS
		 *
		 */
		public function printIconCss(){
			$val        = $this->settings();
			$class      = sanitize_html_class( Smk_Get::value($val, 'title') );
			$icon       = Smk_Get::value($val, 'icon', $this->icon);
			$icon_color = Smk_Get::value($icon, 'color', $this->icon['color']);
			echo '.'. strtolower( $class ) . $this->iconSuffix . '{ color: '. $icon_color .';}';
		}

		################################################################################

		/**
		 * Get view
		 *
		 * @return string Full path
		 */
		public function view($name){
			return SMK_MOD_PATH . 'page-builder/view/'. $name .'.php';
		}

		################################################################################

		/**
		 * Add module to list
		 *
		 * Display module in the list of available modules, PB admin panel
		 *
		 * @return string HTML module
		 */
		public function addModuleMenu(){
			if( $this->mode = 'create' ){
				
				$val      = $this->settings();
				$title    = esc_html( Smk_Get::value($val, 'title') );
				$category = esc_html( Smk_Get::value($val, 'category') );
				$class    = get_option('smk_page_builder_modules');
				$descr    = esc_html( Smk_Get::value($val, 'desc', '&nbsp;') );
				$dotts    = ( strlen($descr) > 28 ) ? '...' : '';

				$icon       = Smk_Get::value($val, 'icon', $this->icon);
				$icon_class = Smk_Get::value($icon, 'class', $this->icon['class']);
				$icon_color = Smk_Get::value($icon, 'color', $this->icon['color']);

				$module = new Smk_Template(
					$this->view('module-list'), 
					array(
						'action'      => $this->id . '_module_form',
						'icon_class'  => sanitize_html_class($icon_class),
						'icon_c_class'=> strtolower( sanitize_html_class($title) ) . $this->iconSuffix,//color class
						'title'       => $title,
						'descr'       => substr( $descr, 0, 28 ) . $dotts,
						'category'    => $category,
						'class'       => array_search(get_class($this), $class),
					)
				);
				$module->render();

			}	
		}

		################################################################################

		/**
		 * Get info field data
		 *
		 * This is mainly used to display a preview text when module is saved.
		 *
		 * @return string HTML
		 */

		public function getInfoField($mod_value_array){
			$mod_sett = $this->settings();
			$info_field = Smk_Get::value($mod_sett, 'info_field', current( array_keys( $this->getTypes() ) ));
			$title_preview = Smk_Get::value($mod_value_array, $info_field, '');
			return ( !is_array($title_preview) ) ? esc_html( strip_tags( $title_preview ) ) : '';
		}
		################################################################################

		/**
		 * Show saved module
		 *
		 * Show a module that is already saved. Used to display the UI when the page is loaded.
		 *
		 * @return string HTML
		 */
		public function showSavedModule($row_key, $section_key, $col_key, $module_key, $module_value){
			$mod_sett = $this->settings();
			$mod_icon = Smk_Get::value($mod_sett, 'icon');
			$mod_icon['class'] = sanitize_html_class( Smk_Get::value($mod_icon, 'class', 'file4') );
			$mod_icon['icon_c_class'] = strtolower( sanitize_html_class( Smk_Get::value($mod_sett, 'title', 'Unknown') ) ) . '-module-icon';
			
			$mod_value_array = ( !empty($module_value) ) ? Smk_Data::decodeJsonE($module_value['val']) : array();
			$title_preview = $this->getInfoField($mod_value_array);
			if( method_exists($this, 'preview') ){
				ob_start();
				$this->preview($mod_value_array);
				$preview = ob_get_contents(); 
				ob_get_clean();
			}
			else{
				$preview = false;
			}
			//Construct module
			$Module = new Smk_Template(
				$this->view('module'), 
				array(
					'name'   => 'smk_page_builder_data[rows]['. $row_key .'][row_sections]['. $section_key .'][column]['. $col_key .'][column_modules]['. $module_key .']',
					'hidden' => array('val', 'class'),
					'key'    => $module_key,
					'value'  => $module_value,
					'icon'   => $mod_icon,
					'type'   => esc_html( Smk_Get::value($mod_sett, 'title', 'Unknown') ),
					'title'  => $title_preview,
					'preview'=> $preview,
					'text'   => array(
						'edit'   => __('Edit','smk_toolkit'),
						'clone'  => __('Clone','smk_toolkit'),
						'delete' => __('Delete','smk_toolkit'),
					)
					
				)
			);
			$Module->render();
			
		}

		################################################################################

		/**
		 * Module form
		 *
		 * This is mainly used to load the form in ajax, but can be used in many ways
		 *
		 * @param array $form Form options array
		 * @param array $saved_data Saved data(if any)
		 * @param string $options_module The module to pass in options switch(optional).
		 * @return string HTML form
		 */
		public function addModuleForm($form, $saved_data = ''){
			echo '<div class="module_info">';
				echo '<div class="module_title">'. $this->getModuleTitle() .'</div>';
				echo '<div class="module_description">'. $this->getModuleDescription() .'</div>';
				// echo '<div class="pb_modal_close">'. __('Cancel', 'smk_toolkit') .'</div>';
			echo '</div>';
			$module = new Smk_ContentBuilder_Form;
			$module->createForm($form, $saved_data);
		}

		################################################################################

		/**
		 * Get defaults from options array.
		 *
		 * @return array
		 */
		public function getDefaults(){
			return Smk_xGet::options_std( $this->form() );
		}

		################################################################################

		/**
		 * Get types from options array.
		 *
		 * @return array
		 */
		public function getTypes(){
			return Smk_xGet::options_type( $this->form() );
		}

		################################################################################

		/**
		 * Get module title.
		 *
		 * @return array
		 */
		public function getModuleTitle(){
			$settings = $this->settings();
			if(array_key_exists('title', $settings)){
				return $settings['title'];
			}
			else{
				return '';
			}
		}

		################################################################################

		/**
		 * Get module description.
		 *
		 * @return array
		 */
		public function getModuleDescription(){
			$settings = $this->settings();
			if(array_key_exists('desc', $settings)){
				return $settings['desc'];
			}
			else{
				return '';
			}
		}

	}//class
}//check class_exists