<?php
/**
 * Smk Panel
 *
 * Designed to create an independent options panel for themes and plugins.
 *
 * -------------------------------------------------------------------------------------
 * Author: Smartik
 * Author URI: http://smartik.ws/
 * Copyright: (c) 2014 Smartik. All rights reserved
 * -------------------------------------------------------------------------------------
 *
 */
if( ! class_exists('Smk_Panel')) {
class Smk_Panel {

	/**
	 * Menu parent
	 *
	 * The slug name for the parent menu (or the file name of a standard WordPress admin page). 
	 * Use `null` or set to 'options.php' if you want to create a page that doesn't appear in any menu. 
	 * 'themes.php', 'tools.php', 'options-general.php', etc.
	 * 
	 */
	public $id;           //Panel id. It MUST be unique. Please use your "special" prefix
	public $parent_slug;  //Parent menu where this submenu will be appended
	public $page_title;   //The page and submenu title
	public $capability;   //User capability. Info: https://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table
	public $fields;       //Form fields
	public $used_in;      //For what it is used? `theme` or `plugin`.
	public $clear;        //Delete everything from DB after plugin deactivation, or theme switch. This is refered only to this panel not to all ceated with SMK Toolkit.
	public $action;       //Custom actions prefix
	public $header_links; //Header links array
	public $footer_links; //Footer links array

	//Plugin register
	public $settings_reg;
	public $plugin_option;

	//------------------------------------//--------------------------------------//
	
	/**
	 * Constructor
	 *
	 * @param $id    string  A unique ID. This term will be used to register settings, and in the URL of this plugin.
	 * @param $args  array   Panel arguments.
	 */
	public function __construct( $id, $args = array() ){

		$defaults = array(
			'parent'     => null, 
			'title'      => false, 
			'capability' => 'activate_plugins',
			'used_in'    => 'plugin',
			'clear'      => false,
			'fields'     => array(),
			'header_links'  => '',
			'footer_links'  => '',
		);

		$args = wp_parse_args($args, $defaults);
		
		$this->parent_slug = ( !empty($args['parent']) ) ? $args['parent'] : null;
		$this->page_title  = ( !empty($args['title']) ) ? $args['title'] : false;
		$this->capability  = ( !empty($args['capability']) ) ? $args['capability'] : 'edit_theme_options';
		$this->id          = ( !empty($id) ) ? 'smk_' . $id : '';
		$this->fields      = ( !empty($args['fields']) ) ? $args['fields'] : array();
		$this->used_in     = ( isset($args['used_in']) && ('theme' && 'plugin' == $args['used_in']) ) ? $args['used_in'] : 'plugin';
		$this->clear       = ( isset($args['clear']) && (true && false == $args['clear']) ) ? $args['clear'] : false;

		//Header/footer links
		$this->header_links = ( isset($args['header_links']) && is_array($args['header_links'] ) ) ? $args['header_links'] : false;

		//Actions prefix
		$this->action = ( !empty($id) ) ? $id : '';

		//Plugin register
		$this->settings_reg = 'smk_plugin_settings_'. $id;
		$this->plugin_option = 'smkop_'. $id;

		//Add menu
		add_action( 'admin_menu', array(&$this, 'menu') );

		//Init
		add_action( 'admin_init', array(&$this, 'panel_init'), 10);
		add_action( 'admin_init', array(&$this, 'diff'), 10);

		//Scripts
		add_action( 'admin_enqueue_scripts', array(&$this, 'enqueue') );
		add_action( 'admin_print_scripts', array(&$this, 'print_scripts'), 90 );
		
		//ajax
		add_action( 'wp_ajax_smk_toolkit_save_opt', array(&$this, 'options_save') );
		add_action( 'wp_ajax_smk_toolkit_reset_opt', array(&$this, 'options_reset') );
		add_action( 'wp_ajax_smk_toolkit_panel_create_css', array(&$this, 'createCss') );

		//Custom hooks. Header, footer links
		add_action($this->action .'_head_links', array(&$this, 'head_links'));
		add_action($this->action .'_footer_links', array(&$this, 'footer_links'));

		// Log
		if( array_key_exists( $this->id, smk_mod_all_panels() ) ){
			smk_log('e', 'A panel with the ID <strong>'. $this->id .'</strong>, already exists. 
				The duplicate name is <strong>'. $this->page_title .'</strong> and the parent is 
				<strong>'. $this->parent_slug .'</strong>
			');
		}

		// Register panel for global access
		add_action('smk_mod_all_panels', array(&$this, 'registerPanelGlobal'));

		// Add this admin panel to Smk Toolkit system info
	/*	$system_info = new Smk_Toolkit_SysInfoPage('smk_toolkit_system_info');
		$system_info->add_sections(array( $this, 'sys_info' ));*/


	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Create dashboard menu
	 */
	function menu(){
		if( $this->page_title ){
			add_submenu_page( $this->parent_slug, $this->page_title, $this->page_title, $this->capability, $this->id, array(&$this, 'admin_page') );
		}
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Enqueue scripts and styles
	 */
	function enqueue() {
		
		if( isset($_GET['page']) ){
			if( $this->id == $_GET['page'] ){
				wp_enqueue_script('jquery-cookie');
				wp_enqueue_script('smk_toolkit_panel_plugins');
			}
		}

	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Print javascript scripts in wp_head for this panel
	 */
	function print_scripts() {
		
		if( isset($_GET['page']) ){
			if( $this->id == $_GET['page'] ){
				do_action($this->id . '_print_scripts');
			}
		}

	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Only fields
	 *
	 * Exclude tabs from array and get only fields
	 *
	 * @return array 
	 */
	function only_fields(){
		return Smk_xGet::unset_tabs( $this->fields );
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get Defaults
	 *
	 * Return only defaults(std) if they are set
	 * 
	 * @return array [id] => val
	 */
	function defaults(){
		return Smk_xGet::options_std( $this->only_fields() );
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Sanitize
	 *
	 * Sanitize data before saving it in DB
	 *
	 * @param $input array Data to be sanitized. This is abstract and is available only on save 
	 * @return array 
	 */
	function sanitize($input){

		// $fields_type = Smk_xGet::options_type_specific( $this->only_fields() );
		// return Smk_Sanitize::options_array($input, $fields_type);
		
		// do_action( 'panel_save_'. $this->id, $this->id );
		
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Reset options
	 *
	 * Reset options to default std. Used in AJAX.
	 *
	 */
	public function options_save(){

		do_action( 'panel_save_'. $this->id, $this->id );
		die();

	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Reset options
	 *
	 * Reset options to default std. Used in AJAX.
	 *
	 */
	public function options_reset(){

		$def = self::defaults();
		update_option($this->plugin_option, $def);
		echo 1;
		die();

	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Admin panel initialization
	 *
	 * Register setting and init the panel.
	 * Note: this DO NOT render the admin page.
	 *
	 */
	function panel_init() {
		register_setting( $this->settings_reg, $this->plugin_option/*, array($this, 'sanitize')*/ );
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Difference.
	 *
	 * Difference between fields and saved data. If the key is not saved add default std.
	 *
	 * @return void It will update the options if there is a difference 
	 */
	public function diff(){

		$default = self::defaults();
		$saved_data = get_option( $this->plugin_option );

		if( $saved_data && is_array($saved_data) && is_array($default) ){
			if( count($saved_data) < count($default) ){
				$merged = array_merge($default, $saved_data);
				update_option($this->plugin_option, $merged);
			}
		}
		elseif( is_array($default) ){
			update_option($this->plugin_option, $default);
		}


	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get view
	 *
	 * Get view by passing the name
	 *
	 * @param $name string 
	 * @return string Full path
	 */
	public function view($name){
		return SMK_MOD_PATH . 'panel/view/'. $name .'.php';
	}

	//------------------------------------//--------------------------------------//

	/**
	 * Admin page
	 *
	 * Render the HTML page
	 *
	 */
	function admin_page(){

		$settings = get_option( $this->plugin_option , self::defaults() );

		$tab_cookie = ( isset($_COOKIE[$this->id]) && $_COOKIE[$this->id] != '' ) ? $_COOKIE[$this->id] : false;
		$tabs = array();
		$tab_isset = 0;
		
		//Build tabs
		foreach ($this->fields as $key => $value) {
			$tab_exists = (isset($value['id']) && !empty($value['id']) ) ? $value['id'] : 'tab' . $key;
			$tab_exists = (isset($value['id']) && $value['id'] == $tab_cookie) ? 1 : 0;
			$tab_isset  = $tab_isset + $tab_exists;

			if( isset($value['type']) && $value['type'] = 'tab' ){
				$tabs[$key] = array(
					'id' =>     isset($value['id']) ? $value['id'] : '',
					'label' =>  isset($value['label']) ? $value['label'] : ' ',
					'fields' => $value['fields'],
					'class' =>  (isset($value['id']) && $value['id'] == $tab_cookie) ? 'active' : '',
					'icon' =>   isset($value['icon']) ? $value['icon'] : '',
				);
			}
		}

		if($tab_isset == 0) {
			$tabs[0]['class'] = 'active';
		}

		$total_tabs = count( $tabs );
		$no_menu = ( $total_tabs < 2 ) ? ' without-menu' : '';

		$main = new Smk_Template(
			$this->view('main'),
			array(
				'class' => array(
					'container' => 'smk_toolkit_plugin_container' . $no_menu,
					'form'      => 'smk_toolkit_plugin_main_form',
					'inner'     => 'panel_inner',
				),
				'form_action' => 'options.php',
				'header' => new Smk_Template(
					$this->view('header'),
					array(
						'page_title' => $this->page_title,
						'header_links_action' => $this->action . '_head_links',
					)
				),
				'footer' => new Smk_Template(
					$this->view('footer'), 
					array(
						'footer_links_action' => $this->action . '_footer_links',
					)
				),
				'top_controls' => new Smk_Template(
					$this->view('top-controls')
				),
				'nav' => new Smk_Template(
					$this->view('nav'), 
					array(
						'id' => $this->id,
						'tabs' => $tabs,
					)
				),
				'options' => new Smk_Template(
					$this->view('options'), 
					array(
						'settings_reg' => $this->settings_reg,
						'plugin_option' => $this->plugin_option,
						'tabs' => $tabs,
						'settings' => $settings,
						'used_in' => $this->used_in,
					)
				),
			)
		);

		$main->render();//Render admin panel
		$this->injectCss();
		// echo $this->getCss();

		// smk_ppprint($this->fields);
		// $all_opts = get_option( $this->plugin_option );
		// smk_ppprint( $all_opts );
		// smk_ppprint( $all_opts['tk_combine_fieldsx1'] );
		// echo '<hr />';
		// smk_ppprint(wp_load_alloptions());
		// smk_ppprint( Smk_xGet::unset_tabs($this->fields) );
		// smk_ppprint( Smk_xGet::options_type_specific( $this->only_fields() ) );


	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Add Head Links
	 *
	 * Inject custom links in the header of this panel
	 *
	 * @return string The HTML
	 */
	public function head_links(){
		$links = $this->header_links;
		if( $links && is_array($links) && count($links) > 0 ){
			foreach ($links as $link) {

				$url    = esc_url( Smk_Get::value( $link, 'url', '#' ) );
				$text   = esc_attr( Smk_Get::value( $link, 'text', '' ) );
				$target = ( isset($link['target']) && in_array( trim( $link['target'] ), array("_blank","_self","_parent","_top") ) ) ? ' target="'. $link['target'] .'"' : '';
				$icon   = sanitize_html_class( Smk_Get::value( $link, 'icon', false ) );
				
				$complete_link = new Smk_Template(
					$this->view('header-link'), 
					array(
						'url' => $url,
						'text' => $text,
						'target' => $target,
						'icon' => $icon,
					)
				);
				$complete_link->render();

			}//foreach
		}
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Add Footer Links
	 *
	 * Inject custom links in the footer of this panel
	 *
	 * @return string The HTML
	 */
	public function footer_links(){
		return false;
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Inject CSS
	 *
	 * Inject custom CSS
	 *
	 * @return string The HTML
	 */
	public function injectCss(){
		return apply_filters( 'smk_luca_opt_inject_css', array() );
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get generated CSS
	 *
	 * Get custom generated CSS
	 *
	 * @return string The CSS
	 */
	public function getCss(){
		
		$css_array = $this->injectCss();
		// var_dump($css_array);
		if( is_array($css_array) && !empty($css_array) ){
			$parser = new Less_Parser();
			foreach ($css_array as $css_mod) {
				if( array_key_exists('type', $css_mod) ){
					if( $css_mod['type'] == ( 'file' || 'plain' ) ){
						switch ($css_mod['type']) {
							case 'file':
								if( isset($css_mod['value']) && isset($css_mod['path']) ){
									if( file_exists($css_mod['value']) ){
										$parser->parseFile( $css_mod['value'], $css_mod['path']);
									}
								}
								break;
							
							case 'plain':
								if( isset($css_mod['value']) ){
									$parser->parse( $css_mod['value'] );
								}
								break;
							
							case 'var':
								if( isset($css_mod['value']) ){
									$parser->ModifyVars( $css_mod['value'] );
								}
								break;
							
							default:
								# code...
								break;
						}
					}
				}
			}
		}
		return $parser->getCss();
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get CSS Path
	 *
	 * Get the path where custom css will be saved
	 *
	 * @return string css path
	 */
	public function getCssPath(){
		
		if( is_admin() ){
			// This website upload dir
			$upload_dir = wp_upload_dir();
			$up_path = trailingslashit($upload_dir['basedir']);
			
			//Call WordPress Filesystem 
			WP_Filesystem();
			global $wp_filesystem;

			// Check if smk-toolkit-css directory exists, if not create it
			if( file_exists($up_path . 'smk-toolkit-css') ){
				if( ! is_dir($up_path . 'smk-toolkit-css') ){
					$wp_filesystem->mkdir($up_path . 'smk-toolkit-css');
				}
			}
			else{
				$wp_filesystem->mkdir($up_path . 'smk-toolkit-css');
			}

			return $up_path . 'smk-toolkit-css/';
		}

	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get CSS File Path
	 *
	 * Get the file path where custom css have been be saved
	 *
	 * @return string css file path
	 */
	public function getCssFilePath(){
		return $this->getCssPath() . 'panel-' . $this->plugin_option . '.css';
	}

	//------------------------------------//--------------------------------------//
	
	/**
	 * Get CSS File URL
	 *
	 * Get the file URL where custom css have been be saved
	 *
	 * @return string css file url
	 */
	public function getCssFileURL(){
		$upload_dir = wp_upload_dir();
		$up_path = trailingslashit($upload_dir['baseurl']);
		return $up_path . 'smk-toolkit-css/panel-' . $this->plugin_option . '.css';
	}
	public function getCssFileURI(){
		return $this->getCssFileURL();
	}
	public static function getStaticCssFileURL($panel_id){
		$upload_dir = wp_upload_dir();
		$up_path = trailingslashit($upload_dir['baseurl']);
		return $up_path . 'smk-toolkit-css/panel-smkop_' . $panel_id . '.css';
	}


	//------------------------------------//--------------------------------------//
	
	/**
	 * Create CSS
	 *
	 * Inject custom when options are saved
	 *
	 * @return void
	 */
	public function createCss(){
		
		if( is_admin() ){
			//Call WordPress Filesystem 
			WP_Filesystem();
			global $wp_filesystem;

			$css = $this->getCss();

			if ( ! $wp_filesystem->put_contents( $this->getCssFilePath(), $css, FS_CHMOD_FILE) ) {
				echo "error saving file!";
			}
			else{
				echo 'Style saved';
			}

		}

		die();
	}

	public function registerPanelGlobal($panels){
		if( ! array_key_exists($this->id, $panels) ){
			$panels[ $this->id ] = array(
				'id'           => $this->id,
				'options_name' => $this->plugin_option,
				'parent_slug'  => $this->parent_slug,
				'page_title'   => $this->page_title,
				'capability'   => $this->capability,
				'used_in'      => $this->used_in,
				'fields'       => $this->fields,
			);
		}
		
		return $panels;
	}


	}//Class end
}//class_exists check end